package VAST::args;
BEGIN {
  $VAST::args::VERSION = '0.04';
}
use utf8;
use strict;
use warnings;
use Mildew::AST::Helpers;

sub emit_m0ld {
    my $m = shift;
    if ($m->{semiarglist}) {
        $m->{semiarglist}->emit_m0ld;
    } elsif ($m->{arglist}[0]) {
        $m->{arglist}[0]->emit_m0ld;
    } else {
        #???
    }
}

1;
